﻿using IcapClientDll;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace IcapClientWinforms
{
    public partial class FormSend : Form
    {
        private IcapConnection icapConnection;

        private string strResponseFilePath;

        private string strRequestFilePath;

        public FormSend()
        {
            InitializeComponent();

            /* Dump status in file */
            string strOutputDirectory = Path.GetFullPath("Dumps");
            if (Directory.Exists(strOutputDirectory) == false)
            {
                Directory.CreateDirectory(strOutputDirectory);
            }

            strResponseFilePath = Path.Combine(strOutputDirectory, string.Format("resp_dump_{0}_{1}.txt", DateTime.Now.ToLongDateString(), DateTime.Now.ToLongTimeString()));
            strRequestFilePath = Path.Combine(strOutputDirectory, string.Format("req_dump_{0}_{1}.txt", DateTime.Now.ToLongDateString(), DateTime.Now.ToLongTimeString()));
            strResponseFilePath = strResponseFilePath.Replace(":", "_").Replace("-", "_");
            strRequestFilePath = strRequestFilePath.Replace(":", "_").Replace("-", "_");
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            textBoxHostname.Text = textBoxHostname.Tag as string;
            textBoxPort.Text = textBoxPort.Tag as string;
            textBoxServiceName.Text = textBoxServiceName.Tag as string;
            comboBoxMethod.SelectedIndex = 0;
            textBoxPreviewSize.Text = textBoxPreviewSize.Tag as string;
            listBoxFiles.Items.Clear();

            textBoxHostname.Focus();
        }

        private void Init()
        {
            comboBoxMethod.SelectedIndex = 0;
        }

        private void textBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !char.IsDigit(e.KeyChar) && !char.IsControl(e.KeyChar);
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            try
            {
                string strResponseText = DateTime.Now.ToLongTimeString() + string.Format(" -- Connecting to Server with hostname - {0} on port number - {1}{2}", textBoxHostname.Text, textBoxPort.Text, Environment.NewLine);
                textBoxResponse.Text += strResponseText;
                File.AppendAllText(strResponseFilePath, strResponseText);

                buttonConnect.Enabled = false;
                Cursor = Cursors.WaitCursor;
                groupBoxInput.Enabled = false;

                /* Try to create a Socket Connection to given Server details */
                icapConnection = new IcapConnection(textBoxHostname.Text, Convert.ToInt32(textBoxPort.Text), textBoxServiceName.Text, comboBoxMethod.SelectedText);

                string strRequestText = DateTime.Now.ToLongTimeString() + " -- " + icapConnection.strOptionsRequestPayload;
                textBoxRequest.Text += strRequestText;
                File.AppendAllText(strRequestFilePath, strRequestText);

                if (icapConnection != null && icapConnection.IsConnected == true)
                {
                    strResponseText = DateTime.Now.ToLongTimeString() + " -- Connected to ICAP Server." + Environment.NewLine + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    MessageBox.Show("Successfully connected to ICAP Server.", "ICAP Connection", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    strResponseText = DateTime.Now.ToLongTimeString() + " -- Failed to connect to ICAP Server." + Environment.NewLine + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    MessageBox.Show("Unable to connect to ICAP server with the credentials you have provided.", "ICAP Connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                fnShowExceptionMessageBox("Connection to ICAP Server", ex);
            }
            finally
            {
                Cursor = Cursors.Default;
                groupBoxInput.Enabled = true;
                buttonConnect.Enabled = true;
            }
        }

        private void fnShowExceptionMessageBox(string strFunction, Exception ex)
        {
            try
            {
                string strErrorMessage = string.Format("Error in {0}: {1}", strFunction, ex.Message);
                MessageBox.Show(strErrorMessage, "ICAP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                File.AppendAllText(strResponseFilePath, strErrorMessage);
            }
            catch (Exception) { }
        }

        private void textBox_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrEmpty(textBoxHostname.Text) == false && string.IsNullOrEmpty(textBoxPort.Text) == false && string.IsNullOrEmpty(textBoxServiceName.Text) == false)
                {
                    buttonConnect.Enabled = true;
                }
                else
                {
                    buttonConnect.Enabled = false;
                }
            }
            catch (Exception) { }
        }

        private void FormSend_Load(object sender, EventArgs e)
        {
            try
            {
                Init();
                textBox_TextChanged(sender, e);
            }
            catch (Exception) { }
        }

        private void buttonSend_Click(object sender, EventArgs e)
        {
            try
            {
                switch (comboBoxMethod.SelectedIndex)
                {
                    case 0:
                        fnSendOptions();
                        break;
                    case 1:
                        fnSendRespMod();
                        break;
                    case 2:
                        fnSendReqMod();
                        break;
                    case 3:
                        fnSendFileMod();
                        break;
                    default:
                        MessageBox.Show("Wrong method selected.", "Wrong method.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        break;
                }
            }
            catch (Exception) { }
            finally
            {
                Cursor = Cursors.Default;
                buttonSend.Enabled = true;
                groupBoxInput.Enabled = true;
            }
        }

        private void buttonBrowse_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult result = folderBrowserDialog.ShowDialog();
                if (result.Equals(DialogResult.OK))
                {
                    if (string.IsNullOrEmpty(folderBrowserDialog.SelectedPath) == false && Directory.Exists(folderBrowserDialog.SelectedPath) == true)
                    {
                        if (listBoxFiles.Items.Count > 0) { listBoxFiles.Items.Clear(); }
                        listBoxFiles.Items.AddRange(Directory.EnumerateFiles(folderBrowserDialog.SelectedPath, "*.", SearchOption.AllDirectories).ToArray());
                    }
                }
            }
            catch (Exception) { }
        }

        private void buttonBrowseFile_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult result = openFileDialog.ShowDialog();
                if (result.Equals(DialogResult.OK))
                {
                    if (openFileDialog.FileNames.Length > 0)
                    {
                        if (listBoxFiles.Items.Count > 0) { listBoxFiles.Items.Clear(); }
                        listBoxFiles.Items.AddRange(openFileDialog.FileNames);
                    }
                }
            }
            catch (Exception) { }
        }

        private void fnSendOptions()
        {
            try
            {
                if (string.IsNullOrEmpty(textBoxHostname.Text) == false && string.IsNullOrEmpty(textBoxPort.Text) == false && string.IsNullOrEmpty(textBoxServiceName.Text) == false && comboBoxMethod.SelectedIndex == 0)
                {
                    buttonSend.Enabled = false;
                    groupBoxInput.Enabled = false;
                    Cursor = Cursors.WaitCursor;

                    string strResponse = DateTime.Now.ToLongTimeString() + " -- Getting ICAP Server Options." + Environment.NewLine;
                    textBoxResponse.Text += strResponse;
                    File.AppendAllText(strResponseFilePath, strResponse);

                    bool bProceed = true;
                    if (icapConnection == null)
                    {
                        strResponse = DateTime.Now.ToLongTimeString() + string.Format(" -- Connecting to Server with hostname - {0} on port number - {1}.....{2}", textBoxHostname.Text, textBoxPort.Text, Environment.NewLine);
                        textBoxResponse.Text += strResponse;
                        File.AppendAllText(strResponseFilePath, strResponse);

                        icapConnection = new IcapConnection(textBoxHostname.Text, Convert.ToInt32(textBoxPort.Text), textBoxServiceName.Text, comboBoxMethod.SelectedText);

                        string strRequestText = DateTime.Now.ToLongTimeString() + " -- " + icapConnection.strOptionsRequestPayload;
                        textBoxRequest.Text += strRequestText;
                        File.AppendAllText(strRequestFilePath, strRequestText);

                        if (icapConnection != null && icapConnection.IsConnected == true)
                        {
                            strResponse = DateTime.Now.ToLongTimeString() + " -- Connected to ICAP Server." + Environment.NewLine + Environment.NewLine;
                            textBoxResponse.Text += strResponse;
                            File.AppendAllText(strResponseFilePath, strResponse);

                            bProceed = true;
                        }
                        else
                        {
                            strResponse = DateTime.Now.ToLongTimeString() + " -- Failed to connect to ICAP Server." + Environment.NewLine + Environment.NewLine;
                            textBoxResponse.Text += strResponse;
                            File.AppendAllText(strResponseFilePath, strResponse);

                            bProceed = false;
                            MessageBox.Show("Unable to connect to ICAP server with the credentials you have provided.", "ICAP Connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }

                    if (bProceed == true)
                    {
                        string strOptions = icapConnection.fnGetServerOptions();

                        textBoxRequest.Text += DateTime.Now.ToLongTimeString() + " -- " + icapConnection.strOptionsRequestPayload;

                        if (string.IsNullOrEmpty(strOptions) == false)
                        {
                            strResponse = DateTime.Now.ToLongTimeString() + " -- " + strOptions + Environment.NewLine + Environment.NewLine;
                            textBoxResponse.Text += strResponse;
                            File.AppendAllText(strResponseFilePath, strResponse);
                        }
                        else
                        {
                            strResponse = DateTime.Now.ToLongTimeString() + " -- " + icapConnection.Error + Environment.NewLine;
                            textBoxResponse.Text += strResponse;
                            File.AppendAllText(strResponseFilePath, strResponse);
                        }
                    }
                }
                else
                {
                    string strResponse = DateTime.Now.ToLongTimeString() + " -- One or more input values missing." + Environment.NewLine;
                    textBoxResponse.Text += strResponse;
                    File.AppendAllText(strResponseFilePath, strResponse);

                    MessageBox.Show("One or more input values are missing. Please check before proceeding.", "Cannot scan", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(strResponseFilePath, ex.Message);
                MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                textBoxResponse.SelectionStart = textBoxResponse.Text.Length;
                textBoxResponse.ScrollToCaret();
            }
        }

        private void fnSendRespMod()
        {
            try
            {
                if (string.IsNullOrEmpty(textBoxHostname.Text) == false && string.IsNullOrEmpty(textBoxPort.Text) == false && string.IsNullOrEmpty(textBoxServiceName.Text) == false && comboBoxMethod.SelectedIndex == 1 && listBoxFiles.Items.Count > 0)
                {
                    buttonSend.Enabled = false;
                    groupBoxInput.Enabled = false;
                    Cursor = Cursors.WaitCursor;

                    string strResponseText = DateTime.Now.ToLongTimeString() + " -- Sending RESPMOD for " + listBoxFiles.Items.Count.ToString() + " files." + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    bool bProceed = true;
                    if (icapConnection == null)
                    {
                        strResponseText = DateTime.Now.ToLongTimeString() + string.Format(" -- Connecting to Server with hostname - {0} on port number - {1}.....{2}", textBoxHostname.Text, textBoxPort.Text, Environment.NewLine);
                        textBoxResponse.Text += strResponseText;
                        File.AppendAllText(strResponseFilePath, strResponseText);

                        icapConnection = new IcapConnection(textBoxHostname.Text, Convert.ToInt32(textBoxPort.Text), textBoxServiceName.Text, comboBoxMethod.SelectedText);

                        string strRequestText = DateTime.Now.ToLongTimeString() + " -- " + icapConnection.strOptionsRequestPayload;
                        textBoxRequest.Text += strRequestText;
                        File.AppendAllText(strRequestFilePath, strRequestText);

                        if (icapConnection != null && icapConnection.IsConnected == true)
                        {
                            strResponseText = DateTime.Now.ToLongTimeString() + " -- Connected to ICAP Server." + Environment.NewLine;
                            textBoxResponse.Text += strResponseText;
                            File.AppendAllText(strResponseFilePath, strResponseText);

                            bProceed = true;
                        }
                        else
                        {
                            strResponseText = DateTime.Now.ToLongTimeString() + " -- Failed to connect to ICAP Server." + Environment.NewLine;
                            textBoxResponse.Text += strResponseText;
                            File.AppendAllText(strResponseFilePath, strResponseText);

                            bProceed = false;
                            MessageBox.Show("Unable to connect to ICAP server with the credentials you have provided.", "ICAP Connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }

                    if (bProceed == true)
                    {
                        List<string> lstFiles = new List<string>();
                        foreach (object temp in listBoxFiles.Items)
                        {
                            lstFiles.Add((string)temp);
                        }

                        List<IcapResponse> response = icapConnection.fnScanFiles_RESPMOD(lstFiles);
                        fnParseResponse(response);
                    }
                }
                else
                {
                    string strResponseText = DateTime.Now.ToLongTimeString() + " -- (RESPMOD) One or more input values missing." + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    MessageBox.Show("One or more input values are missing. Please check before proceeding.", "Cannot scan", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(strResponseFilePath, ex.Message);
                MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                buttonSend.Enabled = true;
                groupBoxInput.Enabled = true;
                Cursor = Cursors.Default;
            }
        }

        private void fnSendReqMod()
        {
            try
            {
                if (string.IsNullOrEmpty(textBoxHostname.Text) == false && string.IsNullOrEmpty(textBoxPort.Text) == false && string.IsNullOrEmpty(textBoxServiceName.Text) == false && comboBoxMethod.SelectedIndex == 2 && listBoxFiles.Items.Count > 0)
                {
                    buttonSend.Enabled = false;
                    groupBoxInput.Enabled = false;
                    Cursor = Cursors.WaitCursor;

                    string strResponseText = DateTime.Now.ToLongTimeString() + " -- Sending REQMOD for " + listBoxFiles.Items.Count.ToString() + " files." + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    bool bProceed = true;
                    if (icapConnection == null)
                    {
                        strResponseText = DateTime.Now.ToLongTimeString() + string.Format(" -- Connecting to Server with hostname - {0} on port number - {1}.....{2}", textBoxHostname.Text, textBoxPort.Text, Environment.NewLine);
                        textBoxResponse.Text += strResponseText;
                        File.AppendAllText(strResponseFilePath, strResponseText);

                        icapConnection = new IcapConnection(textBoxHostname.Text, Convert.ToInt32(textBoxPort.Text), textBoxServiceName.Text, comboBoxMethod.SelectedText);

                        string strRequestText = DateTime.Now.ToLongTimeString() + " -- " + icapConnection.strOptionsRequestPayload;
                        textBoxRequest.Text += strRequestText;
                        File.AppendAllText(strRequestFilePath, strRequestText);

                        if (icapConnection != null && icapConnection.IsConnected == true)
                        {
                            strResponseText = DateTime.Now.ToLongTimeString() + " -- Connected to ICAP Server." + Environment.NewLine;
                            textBoxResponse.Text += strResponseText;
                            File.AppendAllText(strResponseFilePath, strResponseText);

                            bProceed = true;
                        }
                        else
                        {
                            strResponseText = DateTime.Now.ToLongTimeString() + " -- Failed to connect to ICAP Server." + Environment.NewLine;
                            textBoxResponse.Text += strResponseText;
                            File.AppendAllText(strResponseFilePath, strResponseText);

                            bProceed = false;
                            MessageBox.Show("Unable to connect to ICAP server with the credentials you have provided.", "ICAP Connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }

                    if (bProceed == true)
                    {
                        List<string> lstFiles = new List<string>();
                        foreach (object temp in listBoxFiles.Items)
                        {
                            lstFiles.Add((string)temp);
                        }

                        List<IcapResponse> response = icapConnection.fnScanFiles_REQMOD(lstFiles);
                        fnParseResponse(response);
                    }
                }
                else
                {
                    string strResponseText = DateTime.Now.ToLongTimeString() + " -- (REQMOD) One or more input values missing." + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    MessageBox.Show("One or more input values are missing. Please check before proceeding.", "Cannot scan", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(strResponseFilePath, ex.Message);
                MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                buttonSend.Enabled = true;
                groupBoxInput.Enabled = true;
                Cursor = Cursors.Default;
            }
        }

        private void fnSendFileMod()
        {
            try
            {
                if (string.IsNullOrEmpty(textBoxHostname.Text) == false && string.IsNullOrEmpty(textBoxPort.Text) == false && string.IsNullOrEmpty(textBoxServiceName.Text) == false && comboBoxMethod.SelectedIndex == 3 && listBoxFiles.Items.Count > 0)
                {
                    buttonSend.Enabled = false;
                    groupBoxInput.Enabled = false;
                    Cursor = Cursors.WaitCursor;

                    string strResponseText = DateTime.Now.ToLongTimeString() + " -- Sending FILEMOD for " + listBoxFiles.Items.Count.ToString() + " files." + Environment.NewLine;
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    bool bProceed = true;
                    if (icapConnection == null)
                    {
                        strResponseText = DateTime.Now.ToLongTimeString() + string.Format(" -- Connecting to Server with hostname - {0} on port number - {1}.....{2}", textBoxHostname.Text, textBoxPort.Text, Environment.NewLine);
                        textBoxResponse.Text += strResponseText;
                        File.AppendAllText(strResponseFilePath, strResponseText);

                        icapConnection = new IcapConnection(textBoxHostname.Text, Convert.ToInt32(textBoxPort.Text), textBoxServiceName.Text, comboBoxMethod.SelectedText);

                        string strRequestText = DateTime.Now.ToLongTimeString() + " -- " + icapConnection.strOptionsRequestPayload;
                        textBoxRequest.Text += strRequestText;
                        File.AppendAllText(strRequestFilePath, strRequestText);

                        if (icapConnection != null && icapConnection.IsConnected == true)
                        {
                            strResponseText = DateTime.Now.ToLongTimeString() + " -- Connected to ICAP Server." + Environment.NewLine;
                            textBoxResponse.Text += strResponseText;
                            File.AppendAllText(strResponseFilePath, strResponseText);

                            bProceed = true;
                        }
                        else
                        {
                            strResponseText = DateTime.Now.ToLongTimeString() + " -- Failed to connect to ICAP Server." + Environment.NewLine;
                            textBoxResponse.Text += strResponseText;
                            File.AppendAllText(strResponseFilePath, strResponseText);

                            bProceed = false;
                            MessageBox.Show("Unable to connect to ICAP server with the credentials you have provided.", "ICAP Connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }

                    if (bProceed == true)
                    {
                        List<string> lstFiles = new List<string>();
                        foreach (object temp in listBoxFiles.Items)
                        {
                            lstFiles.Add((string)temp);
                        }

                        List<IcapResponse> response = icapConnection.fnScanFiles_FILEMOD(lstFiles);
                        fnParseResponse(response);
                    }
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(strResponseFilePath, ex.Message);
                MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                buttonSend.Enabled = true;
                groupBoxInput.Enabled = true;
                Cursor = Cursors.Default;
            }
        }

        private void FormSend_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                try
                {
                    if (icapConnection != null)
                    {
                        icapConnection.Dispose();
                    }

                    File.AppendAllText(strRequestFilePath.Replace(".txt", "_end.txt"), textBoxRequest.Text);
                    File.AppendAllText(strResponseFilePath.Replace(".txt", "_end.txt"), textBoxResponse.Text);
                }
                catch (Exception) { }
            }
            catch (Exception) { }
        }

        private void comboBoxMethod_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (comboBoxMethod.SelectedIndex == 0)
                {
                    buttonSend.Text = "Send";
                }
                else
                {
                    buttonSend.Text = "Scan";
                }
            }
            catch (Exception) { }
        }

        private void fnParseResponse(List<IcapResponse> responses)
        {
            try
            {
                if (responses != null && responses.Count > 0)
                {
                    string strResponseText = string.Format("{0}{1} -- RESPONSE FROM SERVER{0}", Environment.NewLine, DateTime.Now.ToLongTimeString());
                    textBoxResponse.Text += strResponseText;
                    File.AppendAllText(strResponseFilePath, strResponseText);

                    foreach (IcapResponse response in responses)
                    {
                        string strRequestText = string.Format("{0}{1} -- {2}{0}", Environment.NewLine, DateTime.Now.ToLongTimeString(), response.RequestPaylod);
                        textBoxRequest.Text += strRequestText;
                        File.AppendAllText(strRequestFilePath, strRequestText);

                        strResponseText = response.fnGetString(Environment.NewLine);
                        textBoxResponse.Text += strResponseText;
                        File.AppendAllText(strResponseFilePath, strResponseText);
                    }
                }
                else
                {
                    string strResponseText = "No response received from ICAP server.";
                    MessageBox.Show(strResponseText, "No response", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    File.AppendAllText(strResponseFilePath, strResponseText);
                }
            }
            catch (Exception ex)
            {
                File.AppendAllText(strResponseFilePath, ex.Message);
                MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                buttonSend.Enabled = true;
                groupBoxInput.Enabled = true;
                Cursor = Cursors.Default;
            }
        }

        private void buttonClearAreas_Click(object sender, EventArgs e)
        {
            try
            {
                textBoxRequest.Text = "";
                textBoxResponse.Text = "";
            }
            catch (Exception) { }
        }

        private void textBoxHostname_Leave(object sender, EventArgs e)
        {
            try
            {
                TextBox tb = sender as TextBox;
                if (tb != null)
                {
                    tb.Text = tb.Text.Trim();
                }
            }
            catch (Exception) { }
        }
    }
}
